@@ -33,6 +33,7 @@ from blinter import (
     _collect_set_variables,
     _detect_line_endings,
     _has_multibyte_chars,
+    lint_batch_file,
     read_file_with_encoding,
 )
 
@@ -199,12 +200,12 @@ class TestWarningIssueChecking:
         assert "W015" in issues[0].rule.code
 
     def test_unquoted_variable_in_echo(self) -> None:
-        """Test unquoted variable detection in ECHO command."""
+        """Test unquoted variable in ECHO - should NOT trigger W005 (only IF comparisons do)."""
         set_vars: Set[str] = set()
         issues = _check_warning_issues("echo %VARIABLE%", 1, set_vars, False)
-        # May also trigger W011 (Unicode handling issue for echo command)
+        # W005 is now more conservative - only checks IF string comparisons
         w005_issues = [i for i in issues if i.rule.code == "W005"]
-        assert len(w005_issues) == 1
+        assert len(w005_issues) == 0, "W005 should not flag echo commands"
 
     def test_ping_without_timeout(self) -> None:
         """Test PING without timeout parameter."""
@@ -432,18 +433,6 @@ class TestGlobalFunctionChecking:
         issues = _check_missing_pause(lines)
         assert len(issues) == 0
 
-    def test_check_mixed_variable_syntax(self) -> None:
-        """Test mixed variable syntax detection."""
-        lines = [
-            "echo %STANDARD_VAR%",
-            "echo !DELAYED_VAR!",
-        ]
-        from blinter import _check_mixed_variable_syntax
-
-        issues = _check_mixed_variable_syntax(lines)
-        assert len(issues) == 1
-        assert "W016" in issues[0].rule.code
-
     def test_check_inconsistent_indentation_mixed(self) -> None:
         """Test mixed tabs and spaces in same line."""
         lines = [
@@ -490,7 +479,22 @@ class TestGlobalFunctionChecking:
             "echo line 13",
             "echo line 14",
             "echo line 15",
-            "echo line 16",  # Now 16 lines, should trigger S013
+            "echo line 16",
+            "echo line 17",
+            "echo line 18",
+            "echo line 19",
+            "echo line 20",
+            "echo line 21",
+            "echo line 22",
+            "echo line 23",
+            "echo line 24",
+            "echo line 25",
+            "echo line 26",
+            "echo line 27",
+            "echo line 28",
+            "echo line 29",
+            "echo line 30",
+            "echo line 31",  # Now 31 lines, should trigger S013 with new threshold of 30
         ]
         from blinter import _check_missing_header_doc
 
@@ -512,32 +516,6 @@ class TestGlobalFunctionChecking:
         issues = _check_missing_header_doc(lines)
         assert len(issues) == 0
 
-    def test_check_redundant_assignments(self) -> None:
-        """Test redundant variable assignments detection."""
-        lines = [
-            "set VAR=value1",
-            "set VAR=value2",  # Redundant - no usage between
-            "echo %VAR%",
-        ]
-        from blinter import _check_redundant_assignments
-
-        issues = _check_redundant_assignments(lines)
-        assert len(issues) == 1
-        assert "P011" in issues[0].rule.code
-
-    def test_check_redundant_assignments_with_usage(self) -> None:
-        """Test no warning when variable is used between assignments."""
-        lines = [
-            "set VAR=value1",
-            "echo %VAR%",  # Used here
-            "set VAR=value2",  # Not redundant
-            "echo %VAR%",
-        ]
-        from blinter import _check_redundant_assignments
-
-        issues = _check_redundant_assignments(lines)
-        assert len(issues) == 0
-
 
 class TestMainFunctionEdgeCases:
     """Test main function edge cases and CLI argument handling."""
@@ -696,19 +674,6 @@ class TestAdditionalEdgeCaseScenarios:
         issues = _check_missing_pause(lines)
         assert len(issues) == 0
 
-    def test_mixed_variable_syntax_edge_case(self) -> None:
-        """Test _check_mixed_variable_syntax with delayed variables first."""
-        from blinter import _check_mixed_variable_syntax
-
-        lines = [
-            "echo !DELAYED_VAR!",  # Delayed expansion first
-            "echo %STANDARD_VAR%",  # Standard expansion second
-        ]
-        issues = _check_mixed_variable_syntax(lines)
-        assert len(issues) == 1
-        assert "W016" in issues[0].rule.code
-        # Should flag the standard variable since delayed came first
-
     def test_find_batch_files_not_file_or_directory(self) -> None:
         """Test find_batch_files with invalid path type."""
         import os
@@ -746,20 +711,6 @@ class TestAdditionalEdgeCaseScenarios:
         # Test non-safe context
         assert _is_command_in_safe_context("del *.* /q") is False
 
-    def test_redundant_assignments_edge_case(self) -> None:
-        """Test _check_redundant_assignments with complex patterns."""
-        from blinter import _check_redundant_assignments
-
-        # Test assignment with delayed variable usage
-        lines = [
-            "set VAR=value1",
-            "set VAR=value2",
-            "set VAR=value3",  # Multiple redundant assignments
-            "echo !VAR!",  # Usage with delayed expansion
-        ]
-        issues = _check_redundant_assignments(lines)
-        assert len(issues) == 2  # Two redundant assignments
-
     def test_chardet_detected_encoding_not_in_list(self) -> None:
         """Test chardet detecting encoding not in our default list."""
         from blinter import read_file_with_encoding
@@ -841,21 +792,6 @@ class TestAdditionalEdgeCaseScenarios:
         issues = _check_inconsistent_indentation(lines)
         assert len(issues) == 0  # Should not flag with only one indented line
 
-    def test_redundant_assignments_between_check(self) -> None:
-        """Test redundant assignments checking logic between assignments."""
-        from blinter import _check_redundant_assignments
-
-        # Test the specific branch where assignments have no usage between them
-        lines = [
-            "set VAR1=first",
-            "set VAR2=other",
-            "set VAR1=second",  # Should be flagged - no usage of VAR1 between assignments
-            "set VAR1=third",  # Should also be flagged
-            "echo %VAR1%",  # Finally used
-        ]
-        issues = _check_redundant_assignments(lines)
-        assert len(issues) == 2  # Two redundant assignments for VAR1
-
     def test_main_function_no_path_provided(self) -> None:
         """Test main function when no path is provided."""
         import sys
@@ -966,7 +902,8 @@ class TestWarningChecking:
         """Test detection of older Windows commands."""
         set_vars: set[str] = set()
         # Removed "timeout" as it's been available since Windows Vista
-        for cmd in ["choice", "forfiles", "where", "robocopy", "icacls"]:
+        # Removed "robocopy" as it's been available since Windows Vista/Server 2008 (2007-2008)
+        for cmd in ["choice", "forfiles", "where", "icacls"]:
             issues = _check_warning_issues(f"{cmd} /param", 1, set_vars, False)
             warning_issues = [i for i in issues if i.rule.code == "W009"]
             assert len(warning_issues) == 1
@@ -1446,6 +1383,7 @@ IF EXIST myfile.txt == "yes" ECHO Found
 
         from blinter import lint_batch_file
 
+        # Test truly invalid characters (< > |) are still caught
         content = """@echo off
 COPY "file<test>.txt", "dest>folder",
 """
@@ -1457,6 +1395,19 @@ COPY "file<test>.txt", "dest>folder",
         finally:
             os.unlink(temp_file)
 
+        # Test that wildcards (* and ?) are NOT flagged as invalid
+        content2 = """@echo off
+COPY "*.txt", "dest",
+DEL "file?.bat",
+"""
+        temp_file2 = self.create_temp_batch_file(content2)
+        try:
+            issues2 = lint_batch_file(temp_file2)
+            e005_issues2 = [i for i in issues2 if i.rule.code == "E005"]
+            assert len(e005_issues2) == 0  # Wildcards should NOT be flagged
+        finally:
+            os.unlink(temp_file2)
+
     def test_for_loop_missing_do_e010(self) -> None:
         """Test E010 rule for FOR loop missing DO keyword."""
         import os
@@ -1790,3 +1741,291 @@ class TestSpecializedEdgeCases:
             assert isinstance(issues, list)
         finally:
             os.unlink(temp_path)
+
+
+class TestEnhancedQuoteChecking:
+    """Test cases for comprehensive quote checking implementation."""
+
+    def create_temp_batch_file(self, content: str) -> str:
+        """Helper method to create a temporary batch file with given content."""
+        with tempfile.NamedTemporaryFile(
+            mode="w", suffix=".bat", delete=False, encoding="utf-8"
+        ) as temp_file:
+            temp_file.write(content)
+            return temp_file.name
+
+    def test_caret_escaped_quotes(self) -> None:
+        """Test that caret-escaped quotes (^") are handled correctly."""
+        content = """@ECHO OFF
+ECHO This is a caret-escaped quote: ^"test^"
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Should not report unmatched quotes for caret-escaped quotes
+            quote_issues = [issue for issue in issues if "quote" in issue.rule.name.lower()]
+            assert len(quote_issues) == 0
+        finally:
+            os.unlink(temp_file)
+
+    def test_escaped_quotes_double_quote(self) -> None:
+        """Test that double-quote escaping ("") is handled correctly."""
+        content = """@ECHO OFF
+SET "VAR=This has ""escaped"" quotes"
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Should not report E009 (unmatched quotes) for "" escaping
+            # May report E028 (complex quote escaping) which is a different check
+            e009_issues = [issue for issue in issues if issue.rule.code == "E009"]
+            assert len(e009_issues) == 0
+        finally:
+            os.unlink(temp_file)
+
+    def test_line_continuation_with_quote(self) -> None:
+        """Test that line continuation (^) with quotes is handled correctly."""
+        content = """@ECHO OFF
+ECHO This line continues^
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Should handle line continuation properly
+            quote_issues = [issue for issue in issues if "quote" in issue.rule.name.lower()]
+            # Line continuation should prevent false positive
+            assert len(quote_issues) == 0
+        finally:
+            os.unlink(temp_file)
+
+    def test_delayed_expansion_in_set(self) -> None:
+        """Test that delayed expansion in SET commands is handled correctly."""
+        content = """@ECHO OFF
+SETLOCAL ENABLEDELAYEDEXPANSION
+SET "VAR=!OTHER_VAR!"
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Should not incorrectly flag delayed expansion
+            quote_issues = [issue for issue in issues if "quote" in issue.rule.name.lower()]
+            assert len(quote_issues) == 0
+        finally:
+            os.unlink(temp_file)
+
+    def test_call_label_substitution(self) -> None:
+        """Test that CALL label substitution is handled correctly."""
+        content = """@ECHO OFF
+CALL :subroutine "param1" "param2"
+GOTO :EOF
+
+:subroutine
+ECHO %~1
+EXIT /B
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Should handle CALL label substitution without false positives
+            quote_issues = [issue for issue in issues if "quote" in issue.rule.name.lower()]
+            assert len(quote_issues) == 0
+        finally:
+            os.unlink(temp_file)
+
+    def test_parentheses_context_tracking(self) -> None:
+        """Test that parentheses context is tracked correctly."""
+        content = """@ECHO OFF
+IF EXIST "file.txt" (
+    ECHO "File exists"
+)
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Should correctly handle quotes within parentheses
+            quote_issues = [issue for issue in issues if "quote" in issue.rule.name.lower()]
+            assert len(quote_issues) == 0
+        finally:
+            os.unlink(temp_file)
+
+    def test_actual_unmatched_quote(self) -> None:
+        """Test that actual unmatched quotes are still detected."""
+        content = """@ECHO OFF
+ECHO "This quote is unmatched
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Should detect the unmatched quote
+            quote_issues = [issue for issue in issues if "quote" in issue.rule.name.lower()]
+            assert len(quote_issues) > 0
+        finally:
+            os.unlink(temp_file)
+
+
+class TestEnhancedBlockDetection:
+    """Test cases for comprehensive block detection implementation."""
+
+    def create_temp_batch_file(self, content: str) -> str:
+        """Helper method to create a temporary batch file with given content."""
+        with tempfile.NamedTemporaryFile(
+            mode="w", suffix=".bat", delete=False, encoding="utf-8"
+        ) as temp_file:
+            temp_file.write(content)
+            return temp_file.name
+
+    def test_block_end_with_goto(self) -> None:
+        """Test that GOTO statements properly end blocks."""
+        content = """@ECHO OFF
+REM Documented section
+:main_label
+SET VAR=value
+GOTO :EOF
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Block should end with GOTO
+            doc_issues = [issue for issue in issues if "documentation" in issue.rule.name.lower()]
+            # Should not flag missing documentation after GOTO
+            assert len(doc_issues) == 0 or all(issue.line_number <= 3 for issue in doc_issues)
+        finally:
+            os.unlink(temp_file)
+
+    def test_block_end_with_exit(self) -> None:
+        """Test that EXIT statements properly end blocks."""
+        content = """@ECHO OFF
+REM Documented section
+:subroutine
+ECHO Running subroutine
+EXIT /B 0
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Block should end with EXIT
+            doc_issues = [issue for issue in issues if "documentation" in issue.rule.name.lower()]
+            # Should not flag missing documentation after EXIT
+            assert len(doc_issues) == 0 or all(issue.line_number <= 3 for issue in doc_issues)
+        finally:
+            os.unlink(temp_file)
+
+    def test_block_end_with_new_label(self) -> None:
+        """Test that new labels properly end previous blocks."""
+        content = """@ECHO OFF
+REM First section
+:first_label
+ECHO First section
+
+REM Second section
+:second_label
+ECHO Second section
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Each label should start a new block
+            doc_issues = [issue for issue in issues if "documentation" in issue.rule.name.lower()]
+            # Should properly handle multiple labeled sections
+            assert len(doc_issues) == 0
+        finally:
+            os.unlink(temp_file)
+
+    def test_block_end_with_call_to_label(self) -> None:
+        """Test that CALL to label statements are handled correctly."""
+        content = """@ECHO OFF
+REM Main section
+:main
+CALL :helper
+GOTO :EOF
+
+REM Helper section
+:helper
+ECHO Helper
+EXIT /B
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # CALL to label should be handled
+            doc_issues = [issue for issue in issues if "documentation" in issue.rule.name.lower()]
+            # Should properly handle CALL to label
+            assert len(doc_issues) == 0
+        finally:
+            os.unlink(temp_file)
+
+    def test_block_end_with_closing_parenthesis(self) -> None:
+        """Test that closing parentheses properly end blocks."""
+        content = """@ECHO OFF
+REM Conditional block
+IF EXIST file.txt (
+    ECHO File exists
+    SET VAR=value
+)
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Closing parenthesis should end the block
+            doc_issues = [issue for issue in issues if "documentation" in issue.rule.name.lower()]
+            # Should handle parentheses correctly
+            assert len(doc_issues) == 0
+        finally:
+            os.unlink(temp_file)
+
+    def test_block_end_with_double_empty_lines(self) -> None:
+        """Test that double empty lines properly end blocks."""
+        content = """@ECHO OFF
+REM First section
+FOR %%i IN (1 2 3) DO (
+    ECHO %%i
+)
+
+
+REM This is after double empty lines
+ECHO Done
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Double empty lines should end the block
+            doc_issues = [issue for issue in issues if "documentation" in issue.rule.name.lower()]
+            # Should handle paragraph breaks
+            assert len(doc_issues) == 0
+        finally:
+            os.unlink(temp_file)
+
+    def test_long_block_with_clear_boundary(self) -> None:
+        """Test that long blocks (>15 lines) with clear boundaries are detected."""
+        content = """@ECHO OFF
+REM Long section
+:long_label
+ECHO Line 1
+ECHO Line 2
+ECHO Line 3
+ECHO Line 4
+ECHO Line 5
+ECHO Line 6
+ECHO Line 7
+ECHO Line 8
+ECHO Line 9
+ECHO Line 10
+ECHO Line 11
+ECHO Line 12
+ECHO Line 13
+ECHO Line 14
+ECHO Line 15
+ECHO Line 16
+
+SET VAR=value
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Check for S021 (missing code block documentation) specifically
+            # S013 (file header documentation) is a different check
+            s021_issues = [issue for issue in issues if issue.rule.code == "S021"]
+            # Should handle long blocks properly and not flag S021
+            assert len(s021_issues) == 0
+        finally:
+            os.unlink(temp_file)
