@@ -74,6 +74,26 @@ if %var%==test echo Variable matches
         finally:
             os.unlink(temp_file)
 
+    def test_unquoted_variables_with_numeric_operators(self) -> None:
+        """Test that unquoted variables with numeric comparison operators don't trigger W005."""
+        content = """@echo off
+set errorcode=0
+if %errorcode% equ 0 echo Success
+if %errorcode% neq 1 echo Not one
+if %errorcode% lss 5 echo Less than five
+if %errorcode% leq 10 echo Less or equal ten
+if %errorcode% gtr -1 echo Greater than negative one
+if %errorcode% geq 0 echo Greater or equal zero
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            # W005 should NOT be triggered for numeric operators
+            assert "W005" not in rule_codes
+        finally:
+            os.unlink(temp_file)
+
     def test_goto_without_label(self) -> None:
         """Test detection of goto without matching label."""
         content = """@echo off
@@ -463,10 +483,16 @@ class TestRuleSystem:
             ), f"Rule {code} should have Style severity"
 
         # Check that Security codes have Security severity
+        # Note: SEC006 is an exception - it's STYLE severity (portability issue, not security)
         for code in security_codes:
-            assert (
-                RULES[code].severity == RuleSeverity.SECURITY
-            ), f"Rule {code} should have Security severity"
+            if code == "SEC006":
+                assert (
+                    RULES[code].severity == RuleSeverity.STYLE
+                ), "SEC006 should be STYLE (portability, not security)"
+            else:
+                assert (
+                    RULES[code].severity == RuleSeverity.SECURITY
+                ), f"Rule {code} should have Security severity"
 
         # Check that Performance codes have Performance severity
         for code in performance_codes:
@@ -591,6 +617,8 @@ set /a numeric_var=5
 echo %numeric_var%
 set /p input_var=Enter value:
 echo %input_var%
+REM W005 now only checks IF string comparisons, so add one
+if %var1%==value1 echo match
 """
         temp_file = self.create_temp_batch_file(content)
         try:
@@ -600,8 +628,8 @@ echo %input_var%
             # Should detect undefined variable usage (E006)
             assert "E006" in rule_codes
 
-            # Should detect unquoted variable usage (W005)
-            assert "W005" in rule_codes
+            # Should detect unquoted variable usage in IF comparison (W005)
+            assert "W005" in rule_codes, "W005 should trigger for unquoted IF string comparison"
         finally:
             os.unlink(temp_file)
 
@@ -782,6 +810,23 @@ CALL dir
         finally:
             os.unlink(temp_file)
 
+    def test_call_builtin_commands(self) -> None:
+        """Test CALL to built-in commands like SET (should not trigger E014)."""
+        content = """@echo off
+CALL SET _result=%%%_var%%%
+CALL SET "VAR=value"
+CALL SETLOCAL
+CALL ENDLOCAL
+CALL ECHO Test
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            assert "E014" not in rule_codes  # Should not trigger for built-in commands
+        finally:
+            os.unlink(temp_file)
+
     def test_goto_eof_without_colon(self) -> None:
         """Test E015: Missing colon in GOTO EOF statement."""
         content = """@echo off
